﻿//**************************************************************************
//  Copyright (c) 2011 Igor Timurov, t051200@yandex.ru
//  All Rights Reserved.
//**************************************************************************

using System;
using System.Windows.Forms;

namespace OpenGL
{
    public class GLHelp
    {
        public uint HWND;
        public uint DC;
        public uint RC;

        public bool Available
        {
            get { return DC != 0 && RC != 0; }
        }

        public bool InitializeGL(Control control)
        {
            HWND = (uint)control.Handle.ToInt32();
            DC = WGL.GetDC(HWND);
            // CAUTION: Not doing the following WGL.wglSwapBuffers() on the DC will
            // result in a failure to subsequently create the RC.
            WGL.wglSwapBuffers(DC);

            //Get the pixel format		
            WGL.PIXELFORMATDESCRIPTOR pfd = new WGL.PIXELFORMATDESCRIPTOR();
            WGL.ZeroPixelDescriptor(ref pfd);
            pfd.nVersion = 1;
            pfd.dwFlags = (WGL.PFD_DRAW_TO_WINDOW | WGL.PFD_SUPPORT_OPENGL | WGL.PFD_DOUBLEBUFFER);
            pfd.iPixelType = (byte)(WGL.PFD_TYPE_RGBA);
            pfd.cColorBits = 32;
            pfd.cDepthBits = 32;
            pfd.iLayerType = (byte)(WGL.PFD_MAIN_PLANE);

            int pixelFormatIndex = 0;
            pixelFormatIndex = WGL.ChoosePixelFormat(DC, ref pfd);
            if (pixelFormatIndex == 0)
            {
                MessageBox.Show("Unable to retrieve pixel format");
                return false;
            }

            if (WGL.SetPixelFormat(DC, pixelFormatIndex, ref pfd) == 0)
            {
                MessageBox.Show("Unable to set pixel format");
                return false;
            }
            //Create rendering context
            RC = WGL.wglCreateContext(DC);
            if (RC == 0)
            {
                MessageBox.Show("Unable to get rendering context");
                return false;
            }
            if (WGL.wglMakeCurrent(DC, RC) == 0)
            {
                MessageBox.Show("Unable to make rendering context current");
                return false;
            }
            return true;
        }

        public void ResizeGL(int width, int height)
        {
            if (!Available)
                return;

            if (width == 0 || height == 0)
                return;

            WGL.wglMakeCurrent(DC, RC);
            GL.glViewport(0, 0, width, height);

            GL.glMatrixMode(GL.GL_PROJECTION);
            GL.glLoadIdentity();
            GLU.gluPerspective(60.0, ((double)(width) / (double)(height)), 0.1, 100000.0);

            GL.glMatrixMode(GL.GL_MODELVIEW);
            GL.glLoadIdentity();
        }
    }
}