//**************************************************************************
//  Copyright (c) 2011 Igor Timurov, t051200@yandex.ru
//  All Rights Reserved.
//**************************************************************************

#ifndef __FCOLOR__HED__
#define __FCOLOR__HED__

#include "FBase.h"
#include "FAlgorithm.h"

inl 
void GrayTo(UByte* src, UByte v)
{
	src[0] = v;
	src[1] = v;
	src[2] = v;
}

inl 
UByte ToGray(const UByte* src)
{
	// 11% blue, 30% red, 59% green
	return (UByte)(((UShort)src[0] * 28 + (UShort)src[1] * 150 + (UShort)src[2] * 77) >> 8);
	//return (UByte)(((UShort)src[0] * 85 + (UShort)src[1] * 85 + (UShort)src[2] * 85) >> 8);
}

inl 
UShort UShortToGray(const UShort* src)
{
	// 11% blue, 30% red, 59% green
    return (UShort)(((ULong)src[0] * 7209 + (ULong)src[1] * 38666 + (ULong)src[2] * 19660) >> 16);
}

inl 
void HSLtoRGB(float H, float S, float L, float& R, float& G, float& B)
{
	float Q;
	if(L < 0.5)
		Q = L * (1.0f + S);
	else
		Q = L + S - L * S;
	float P = 2.0f * L - Q;
	float T[3] = {H + 1.0f/3.0f, H, H - 1.0f/3.0f};
	for(int i = 0; i < 3; ++i)
	{
		float& Tc = T[i];
		Tc = Tc < 0 ? Tc + 1.0f : Tc > 1.0f ? Tc - 1.0f : Tc;
		if (Tc < 1.0f/6.0f)
			Tc = P + (Q - P) * Tc * 6.0f;
		else if(Tc < 1.0f/2.0f)
			Tc = Q;
		else if(Tc < 2.0f/3.0f)
			Tc = P + (Q - P) * (2.0f/3.0f - Tc) * 6.0f;
		else
			Tc = P;
	}
	R = T[0];
	G = T[1];
	B = T[2];
}

inl 
void HSLtoRGB(float H, float S, float L, UByte* src)
{
	float r, g, b;
	HSLtoRGB(H, S, L, r, g, b);
	src[0] = (UByte)(r*255.0f);
	src[1] = (UByte)(g*255.0f);
	src[2] = (UByte)(b*255.0f);
}

inl 
void HToRGB(UByte* src, float H) 
{
	float r, g, b;
	HSLtoRGB(H, 1.0, 0.5, r, g, b);
	src[0] = (UByte)(r*255.0f);
	src[1] = (UByte)(g*255.0f);
	src[2] = (UByte)(b*255.0f);
}

inl 
void GToRGB(UByte* src, UByte G) 
{
	HToRGB(src, (float)G/255.0f);
}

inl 
void DirectToRGB(float x, float y, UByte* c)
{
    float h = 0;
    float len = sqrt(x*x + y*y);
    if (len != 0)
    {
        if (x > 0)
            h = acos(y/len);
        else
            h = ((float)FDF_2PI-acos(y/len));
        h = Clamp01(h / (float)FDF_2PI);
    }
    HSLtoRGB(h, len, 0.5, c);
}

inl 
void DirectHToRGB(float z, UByte* c)
{
    c[0] = 0;
    c[1] = 0;
    c[2] = 0;
    if(z > 0)
    {
        c[1] = (UByte)(Clamp01(1-z)*255.0f);
        c[2] = c[1];
    }
    else
    {
        c[0] = (UByte)(Clamp01(1+z)*255.0f);
    }
}
#endif //__FCOLOR__HED__