//**************************************************************************
//  Copyright (c) 2011 Igor Timurov, t051200@yandex.ru
//  All Rights Reserved.
//**************************************************************************

#ifndef __FVECTOR3__HED__
#define __FVECTOR3__HED__

#include "FBase.h"

#pragma pack(push,1)
struct FVector3
{
    float X;
    float Y;
    float Z;

    FVector3():
        X(0), Y(0), Z(0)
    {}

    FVector3(const FVector3& in):
        X(in.X), Y(in.Y), Z(in.Z)
    {}

    FVector3(float in_X, float in_Y, float in_Z):
        X(in_X), Y(in_Y), Z(in_Z)
    {}

    inl FVector3& operator = (const FVector3& in)
    {
        X = in.X;
        Y = in.Y;
        Z = in.Z;
        return *this;
    }

    inl FVector3& operator += (const FVector3& in)
    {
        X += in.X;
        Y += in.Y;
        Z += in.Z;
        return *this;
    }

    inl FVector3& operator -= (const FVector3& in)
    {
        X -= in.X;
        Y -= in.Y;
        Z -= in.Z;
        return *this;
    }

    inl FVector3& operator += (float d)
    {
        X += d;
        Y += d;
        Z += d;
        return *this;
    }

    inl FVector3& operator -= (float d)
    {
        X -= d;
        Y -= d;
        Z -= d;
        return *this;
    }

    inl FVector3& operator *= (float d)
    {
        X *= d;
        Y *= d;
        Z *= d;
        return *this;
    }

    inl FVector3& operator /= (float d)
    {
        return *this *= 1.0f/d;
    }

    inl float SqrMagnitude()
    {
        return X*X + Y*Y + Z*Z;
    }
    
    inl float Magnitude()
    {
        return sqrt(SqrMagnitude());
    }

    inl void Normalize()
    {
        *this /= Magnitude();
    }

    inl void Rotate(const FVector3& v, float f)
    {
	    FVector3 dz(v);
        dz *= Dot(v, *this);
	    FVector3 dx(*this);
        dx -= dz;
	    FVector3 dy;
        Cross(dx, v, dy);
        dx *= cos(f);
        dy *= sin(f);
	    *this = dx;
	    *this += dy;
	    *this += dz;
    }

    inl static float Dot(const FVector3& a, const FVector3& b)
    {
        return a.X*b.X + a.Y*b.Y + a.Z*b.Z;
    }

    inl static void Cross(const FVector3& a, const FVector3& b, FVector3& c)
    {
         float X = a.Y*b.Z - a.Z*b.Y;
         float Y = a.Z*b.X - a.X*b.Z;
         float Z = a.X*b.Y - a.Y*b.X;
         c.X = X;
         c.Y = Y;
         c.Z = Z;
    }
};

#pragma pack(pop)

#endif //__FVECTOR3__HED__