﻿//**************************************************************************
//  Copyright (c) 2011 Igor Timurov, t051200@yandex.ru
//  All Rights Reserved.
//**************************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using OpenGL;

namespace SurfaceScanner
{
    public partial class GLView : UserControl
    {
        GLHelp glhelp = new GLHelp();
        Point mouse = new Point();
        float distance;
        float directH = 500;
        float directA = 0;
        uint surface = 0;

        float[] LightAmbient = { 0.5f, 0.5f, 0.5f, 1.0f }; // Значения фонового света ( НОВОЕ )
        float[] LightDiffuse = { 1.0f, 1.0f, 1.0f, 1.0f }; // Значения диффузного света ( НОВОЕ )
        float[] LightPosition = { 0.0f, -1.0f, 0.0f};// Позиция света ( НОВОЕ )

        public bool bRender = true;           // Флаг режима отображения полигонов
        bool IsInit = false;
        float[] matrix;
        int width;
        int height;
        float scale;


        public GLView()
        {
            InitializeComponent();

            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            this.SetStyle(ControlStyles.DoubleBuffer, false);
            this.SetStyle(ControlStyles.Opaque, true);
            this.SetStyle(ControlStyles.ResizeRedraw, true);
            this.SetStyle(ControlStyles.UserPaint, true);
        }

        public void Init(float[] matrix, int width, int height, float scale)
        {
            this.matrix = matrix;
            this.width = width;
            this.height = height;
            this.scale = scale;
            distance = width / 2;
            IsInit = true;
        }

        void Draw()
        {
            WGL.wglMakeCurrent(glhelp.DC, glhelp.RC);

            if (IsInit)
                DrawGLScene();

            WGL.wglSwapBuffers(glhelp.DC);
        }


        public void ReDraw()
        {
            Invalidate();
        }

        #region Draw sccene

        void Camera()
        {
            GLU.gluLookAt(
                distance,
                directH,
                distance, 
                0, 0, 0, 0, 1, 0);  // Определяет вид и положение камеры
            GL.glRotatef(directA, 0, 1, 0);
        }

        void DrawGLScene()                     // Здесь содержится код рисования
        {
            GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);  // Очистка экрана и буфера глубины
            GL.glLoadIdentity();                       // Сброс просмотра

            Light();
            // Положение   Вид    Вектор вертикали
            Camera();

            RenderHeightMap();                      // Визализация карты высот
            GL.glFlush();
        }

        void Light()
        {
            float[] light0_ambient = { 1.0f, 1.0f, 1.0f };
            float[] light0_diffuse = { 1.0f, 1.0f, 1.0f };
            float[] light0_direction = {0.0f, 0.0f, 1.0f, 1.0f};

            GL.glEnable(GL.GL_LIGHT0);
            GL.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT, light0_ambient);
            GL.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, light0_diffuse);
            GL.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, light0_direction);
        }

        public void BuildSurface()
        {
            GL.glNewList(surface, GL.GL_COMPILE);

            if (bRender)
                GL.glBegin(GL.GL_QUADS);
            else
                GL.glBegin(GL.GL_LINES);

            for (int x = 0; x < width-1; ++x)
            {
                for (int y = 0; y < height-1; ++y)
                {

                    float h1 = GetHeight(x, y) * scale;
                    float h2 = GetHeight(x, y + 1) * scale;
                    float h3 = GetHeight(x + 1, y + 1) * scale;
                    float h4 = GetHeight(x + 1, y) * scale;

                    float sx = h4 - h1;
                    float sz = h2 - h1;

                    GL.glNormal3f(-sx, 1.0f, -sz);

                    SetVertexColor(x, y);
                    GL.glVertex3f(x, h1, y);

                    SetVertexColor(x, y);
                    GL.glVertex3f(x, h2, y + 1);

                    SetVertexColor(x, y);
                    GL.glVertex3f(x + 1, h3, y + 1);

                    SetVertexColor(x, y);
                    GL.glVertex3f(x + 1, h4, y);
                }
            }
            GL.glEnd();

            GL.glEndList();
        }

        void RenderHeightMap()     // Визуализация карты высоты с помощью квадратов
        {
            GL.glTranslated(-width/2, 0, -height/2);

            GL.glCallList(surface);

            GL.glColor4f(1.0f, 1.0f, 1.0f, 1.0f);  // Сбрасываем цвет
        }

        // Эта функция устанавливает значение цвета для конкретного номера, зависящего от номера высоты
        void SetVertexColor(int x, int y)
        {
            //float fColor = GetHeight(x, y);

            // Присвоить оттенок синего цвета для конкретной точки
            GL.glColor3f(1.0f, 1.0f, 1.0f);
        }

        float GetHeight(int x, int y)  // Возвращает высоту из карты вершин (?)
        {
            return matrix[y * width + x];  // Возвращаем значение высоты
        }

        #endregion

        #region Events

        protected override void OnPaintBackground(PaintEventArgs e)
        {
        }

        protected override void OnPaint(System.Windows.Forms.PaintEventArgs e)
        {
            if (!glhelp.Available)
                return;

            Draw();
        }

        private void GLView_SizeChanged(object sender, EventArgs e)
        {
            glhelp.ResizeGL(this.ClientSize.Width, this.ClientSize.Height);
        }

        private void GLView_Load(object sender, EventArgs e)
        {
            glhelp.InitializeGL(this);

            surface = GL.glGenLists(1);
            BuildSurface();

            //Set up OpenGL related characteristics
            glhelp.ResizeGL(this.ClientSize.Width, this.ClientSize.Height);

            GL.glShadeModel(GL.GL_SMOOTH);    // разрешить плавное цветовое сглаживание
            GL.glClearColor(0.0f, 0.0f, 0.0f, 0.5f); // Очистка экрана черным цветом
            GL.glClearDepth(1.0f);
            GL.glEnable(GL.GL_DEPTH_TEST); // Включить буфер глубины
            GL.glDepthFunc(GL.GL_LEQUAL); // Тип теста глубины
            //GL.glHint(GL.GL_PERSPECTIVE_CORRECTION_Hint, GL.GL_NICEST);  // Улучшенные вычисления перспективы

            //GL.glLightfv(GL.GL_LIGHT1, GL.GL_AMBIENT, LightAmbient);    // Установка Фонового Света
            //GL.glLightfv(GL.GL_LIGHT1, GL.GL_DIFFUSE, LightDiffuse);    // Установка Диффузного Света
            //GL.glLightfv(GL.GL_LIGHT1, GL.GL_POSITION, LightPosition);   // Позиция света

            //GL.glEnable(GL.GL_LIGHT1); // Разрешение источника света номер один
            //GL.glEnable(GL.GL_LIGHTING);

            // рассчет освещения
            GL.glEnable(GL.GL_LIGHTING);
            // двухсторонний расчет освещения
            GL.glLightModelf(GL.GL_LIGHT_MODEL_TWO_SIDE, GL.GL_TRUE);
            // автоматическое приведение нормалей к
            // единичной длине
            GL.glEnable(GL.GL_NORMALIZE);
        }

        private void GLView_KeyDown(object sender, KeyEventArgs e)
        {
        }

        private void GLView_KeyUp(object sender, KeyEventArgs e)
        {
        }

        private void GLView_MouseMove(object sender, MouseEventArgs e)
        {
            Point delta = Point.Subtract(e.Location, new Size(mouse));
            if (e.Button == MouseButtons.Left)
            {
                directA += delta.X;
                distance += delta.Y;
            }
            if (e.Button == MouseButtons.Right)
            {
                directH += delta.Y;
            }
            mouse = e.Location;
            ReDraw();
        }

        #endregion
    }
}
