﻿//**************************************************************************
//  Copyright (c) 2011 Igor Timurov, t051200@yandex.ru
//  All Rights Reserved.
//**************************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace SurfaceScanner
{
    public partial class MainForm : Form
    {
        private OpenFileDialog openImage;
        private SaveFileDialog saveRaw;
        private SaveFileDialog saveFile;
        private OpenFileDialog openFile;
        Scanner scanner = new Scanner();
        ScannerImage current = null;

        public MainForm()
        {
            this.openImage = new OpenFileDialog();
            this.openImage.Filter = "Bitmap (*.bmp)|*.bmp|" + "PNG (*.png)|*.png|" + "Jpeg (*.jpg)|*.jpg|" + "All Format|*.*";
            this.openImage.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath);
            this.openImage.FilterIndex = 1;
            this.openImage.Multiselect = true;
            this.openImage.RestoreDirectory = true;

            this.saveFile = new SaveFileDialog();
            this.saveFile.Filter = "SC (*.xml)|*.xml";
            this.saveFile.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath);
            this.saveFile.FilterIndex = 1;
            this.saveFile.RestoreDirectory = true;

            this.saveRaw = new SaveFileDialog();
            this.saveRaw.Filter = "RAW (*.raw)|*.raw";
            this.saveRaw.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath);
            this.saveRaw.FilterIndex = 1;
            this.saveRaw.RestoreDirectory = true;

            this.openFile = new OpenFileDialog();
            this.openFile.Filter = "SC (*.xml)|*.xml";
            this.openFile.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath);
            this.openFile.FilterIndex = 1;
            this.openFile.RestoreDirectory = true;

            InitializeComponent();
            text_box_normal.Text = "0 1 2";
            text_box_correct.Text = "100";
            ShowCurrect();

            //new ViewForm(Scanner.Sub(
            //    @"F:\Projects\FrozenVPS\cs\SurfaceScanner\Debug\.bmp",
            //    @"F:\Projects\FrozenVPS\cs\SurfaceScanner\Debug\01.bmp")).Show();

            list_box_view.Items.Add(new ViewFormParam("Gray", delegate() 
            {
                if (current == null)
                    return;
                new ViewForm(scanner.GetGray(current.Id)).Show();
            }));
            list_box_view.Items.Add(new ViewFormParam("Normal Map", delegate() 
            {
                new ViewForm(scanner.GetNormal()).Show();
            }));
            list_box_view.Items.Add(new ViewFormParam("Normal Dxy", delegate()
            {
                new ViewForm(scanner.GetNormalDxy()).Show();
            }));
            list_box_view.Items.Add(new ViewFormParam("Normal Dz", delegate() 
            {
                new ViewForm(scanner.GetNormalDz()).Show();
            }));
            list_box_view.Items.Add(new ViewFormParam("Height", delegate() 
            {
                new ViewForm(scanner.GetHeight()).Show();
            }));
            list_box_view.Items.Add(new ViewFormParam("Save RAW", delegate() 
            {
                if (saveRaw.ShowDialog(this) == DialogResult.OK)
                {
                    try
                    {
                        scanner.SaveRaw(saveRaw.FileName);
                    }
                    catch (Exception exc)
                    {
                        MessageBox.Show(exc.Message);
                    }
                }
            }));
            list_box_view.Items.Add(new ViewFormParam("Error", delegate()
            {
                new ViewForm(scanner.GetError()).Show();
            }));
            list_box_view.Items.Add(new ViewFormParam("Height 3D", delegate()
            {
                float[] matrix;
                int width;
                int height;
                float scale;
                scanner.GetHeightData(out matrix, out width, out height, out scale);
                new GLForm(matrix, width, height, scale).Show();
            }));
        }

        public class ViewFormParam
        {
            public delegate void ViewFormMethod();
            public string Name;
            public ViewFormMethod Method;

            public ViewFormParam(string name, ViewFormMethod method)
            {
                this.Name = name;
                this.Method = method;
            }

            public override string ToString()
            {
                return Name;
            }
        }

        void ShowCurrect()
        {
            if (current == null)
            {
                label_image_id.Text = "Id: undefined";
                label_image_path.Text = "Path: undefined";
                label_image_direct.Text = "Direct: undefined";
                picture_box.Image = null;
                text_box_rect.Text = "X Y Width Height";
            }
            else
            {
                label_image_id.Text = "Id: " + current.Id;
                label_image_path.Text = "Path: " + current.Path;
                label_image_direct.Text = string.Format("Direct: {{{0:0.000}; {1:0.000}; {2:0.000}}}", current.direct.X, current.direct.Y, current.direct.Z);
                picture_box.Image = current.Bmp;
                text_box_rect.Text = scanner.GetRect();
                text_box_direct_x.Text = current.direct.X.ToString();
                text_box_direct_y.Text = current.direct.Y.ToString();
                text_box_direct_z.Text = current.direct.Z.ToString();
            }
        }

        #region Events

        private void MainForm_Load(object sender, EventArgs e)
        {

        }

        private void button_apply_rect_Click(object sender, EventArgs e)
        {
            scanner.SetRect(text_box_rect.Text);
            text_box_rect.Text = scanner.GetRect();
        }

        private void button_next_image_Click(object sender, EventArgs e)
        {
            foreach (ScannerImage i in scanner.Images)
            {
                if (current == i)
                    current = null;
                else if (current == null)
                    current = i;
            }
            ShowCurrect();
        }

        private void combo_box_image_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        #region Buttons projects

        private void button_new_Click(object sender, EventArgs e)
        {
            if (scanner != null)
                scanner.Dispose();
            scanner = new Scanner();
            current = null;
            ShowCurrect();
        }

        private void button_open_Click(object sender, EventArgs e)
        {
            if (openFile.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    if (scanner != null)
                        scanner.Dispose();
                    scanner = Scanner.Load(openFile.FileName);
                    current = null;
                    ShowCurrect();
                }
                catch (Exception exc)
                {
                    MessageBox.Show(exc.Message);
                }
            }
        }

        private void button_save_Click(object sender, EventArgs e)
        {
            if (saveFile.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    scanner.Save(saveFile.FileName);
                }
                catch (Exception exc)
                {
                    MessageBox.Show(exc.Message);
                }
            }
        }

        private void button_add_image_Click(object sender, EventArgs e)
        {
            if (openImage.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    foreach (string path in openImage.FileNames)
                        scanner.LoadImage(path);
                }
                catch (Exception exc)
                {
                    MessageBox.Show(exc.Message);
                }
            }
        }

        private void button_meter_Click(object sender, EventArgs e)
        {
            if (current == null)
                return;

            new MeterForm(current.Bmp).Show();
        }

        #endregion

        private void button_create_Click(object sender, EventArgs e)
        {
            scanner.Create();
        }

        private void button_create_height_Click(object sender, EventArgs e)
        {
            scanner.MakeHeight();
        }

        private void button_create_normal_Click(object sender, EventArgs e)
        {
            scanner.MakeNormal(text_box_normal.Text);
        }

        private void button_correct_normal_Click(object sender, EventArgs e)
        {
            int m = int.Parse(text_box_correct.Text);
            ProgressForm form = new ProgressForm(delegate(ref int minv, ref int maxv, ref int curv)
            {
                minv = 0;
                maxv = m;
                scanner.CorrectNormal(ref curv, maxv);
            });
            form.ShowDialog(this);
        }

        private void button_angle_apply_Click(object sender, EventArgs e)
        {
            if (current == null)
                return;
            float angleX = float.Parse(text_box_angle_x.Text, Program.FormatProv);
            float angleY = float.Parse(text_box_angle_y.Text, Program.FormatProv);
            ScannerLib.DirectByAngle(angleX, angleY, ref current.direct);
            ShowCurrect();
        }

        private void button_vector_apply_Click(object sender, EventArgs e)
        {
            if (current == null)
                return;
            float vectorX = float.Parse(text_box_vector_x.Text, Program.FormatProv);
            float vectorY = float.Parse(text_box_vector_y.Text, Program.FormatProv);
            float magnitude = float.Parse(text_box_magnitude.Text, Program.FormatProv);
            ScannerLib.DirectByVector(vectorX, vectorY, magnitude, ref current.direct);
            ShowCurrect();
        }

        private void button_apply_direct_Click(object sender, EventArgs e)
        {
            if (current == null)
                return;
            current.direct.X = float.Parse(text_box_direct_x.Text, Program.FormatProv);
            current.direct.Y = float.Parse(text_box_direct_y.Text, Program.FormatProv);
            current.direct.Z = float.Parse(text_box_direct_z.Text, Program.FormatProv);
            ScannerLib.DirectByDirect(ref current.direct);
            ShowCurrect();
        }

        private void list_box_view_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (list_box_view.SelectedIndex != -1)
            {
                ViewFormParam p = list_box_view.SelectedItem as ViewFormParam;
                if (p != null)
                {
                    p.Method();
                }
            }
        }

        #endregion
    }
}
