﻿//**************************************************************************
//  Copyright (c) 2011 Igor Timurov, t051200@yandex.ru
//  All Rights Reserved.
//**************************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;

namespace SurfaceScanner
{
    public partial class MeterForm : Form
    {
        Bitmap buffer;
        Bitmap image;
        Point shift = new Point();
        Point mouse = new Point();

        bool isLine = false;
        Point begin = new Point();
        Point end = new Point();

        public MeterForm(Bitmap image)
        {
            this.image = image;
            InitializeComponent();
            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            this.SetStyle(ControlStyles.DoubleBuffer, false);
            this.SetStyle(ControlStyles.Opaque, true);
            this.SetStyle(ControlStyles.ResizeRedraw, true);
            this.SetStyle(ControlStyles.UserPaint, true);
        }

        void Draw()
        {
            if (buffer != null)
            {
                Graphics g = Graphics.FromImage(buffer);
                g.CompositingMode = CompositingMode.SourceCopy;
                g.CompositingQuality = CompositingQuality.HighSpeed;
                g.InterpolationMode = InterpolationMode.NearestNeighbor;
                g.PixelOffsetMode = PixelOffsetMode.None;
                g.PageUnit = GraphicsUnit.Pixel;

                g.Clear(Color.White);
                Draw(g);
                g.Dispose();
            }
        }

        void Draw(Graphics g)
        {
            g.DrawImage(image, shift);
            g.DrawLine(Pens.Red, begin, end);
        }

        void ReDraw()
        {
            Draw();
            Invalidate();
        }

        protected override void OnResize(EventArgs e)
        {
            buffer = new Bitmap(this.ClientSize.Width, this.ClientSize.Height);
            Draw();
            base.OnResize(e);
        }

        protected override void OnPaint(PaintEventArgs paintEventArgs)
        {
            Graphics g = paintEventArgs.Graphics;
            g.CompositingMode = CompositingMode.SourceCopy;
            g.CompositingQuality = CompositingQuality.HighSpeed;
            g.InterpolationMode = InterpolationMode.NearestNeighbor;
            g.PixelOffsetMode = PixelOffsetMode.None;
            g.PageUnit = GraphicsUnit.Pixel;

            if (buffer != null)
            {
                //Rectangle rect = new Rectangle(new Point(), buffer.Size);
                g.DrawImage(buffer, Point.Empty);
                //g.DrawImage(buffer, rect, rect, GraphicsUnit.Pixel);
            }
            else
            {
                g.Clear(Color.Gray);
            }

        }

        protected override void OnPaintBackground(PaintEventArgs e)
        {
        }

        private void LineForm_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                shift = Point.Add(shift, new Size(Point.Subtract(e.Location, new Size(mouse))));
                ReDraw();
            }
            if (e.Button == MouseButtons.Left)
            {
                end = e.Location;
                ReDraw();
            }
            mouse = e.Location;
        }

        private void LineForm_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                begin = e.Location;
            }
        }

        private void LineForm_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                end = e.Location;
                Point vector = Point.Subtract(end, new Size(begin));
                double magnitude = Math.Sqrt((double)(vector.X * vector.X + vector.Y * vector.Y));
                MessageBox.Show(string.Format("Magnitude: {0:0.00000}\nVectorX: {1}\nVectorY: {2}", magnitude, vector.X, vector.Y), "Line info");
            }
        }
    }
}
